@@ -7,6 +7,7 @@ import os
 
 from PIL import Image
 from sqlalchemy.orm import eagerload
+from sqlalchemy import func
 from time import time
 from webhelpers.feedgenerator import Rss201rev2Feed
 
@@ -380,6 +381,15 @@ def preview_article(request):
     
     return Response(body=complete, content_type='text/html')
 
+def _update_comments_counters(dbsession, article):
+    """
+    Re-count total and approved comments and update corresponding counters for the article
+    """
+    approved_cnt = dbsession.query(func.count(Comment.id)).filter(Comment.article==article).filter(Comment.is_approved==True).scalar()
+    total_cnt = dbsession.query(func.count(Comment.id)).filter(Comment.article==article).scalar()
+    article.comments_approved = approved_cnt
+    article.comments_total = total_cnt
+
 def _view_article(request, article_id=None, article=None):
     c = dict()
     
@@ -411,11 +421,15 @@ def _view_article(request, article_id=None, article=None):
 
     scope = dict(thread=[])
 
+    # we should hide all not approved comments for everyone who isn't a site admin
+    display_not_approved = user.has_permission('admin')
     def build_thread(parent_id, indent):
         if parent_id not in comments_dict:
             return
 
         for x in comments_dict[parent_id]:
+            if not display_not_approved and not x.is_approved:
+                continue
             setattr(x, '_indent', indent)
             scope['thread'].append(x)
             build_thread(x.id, indent+1)
@@ -458,6 +472,9 @@ def view_article(request):
     if not user.has_permission('edit_article'):
         q = q.filter(Article.is_draft==False)
     article = q.first()
+
+    if article is None:
+        return HTTPNotFound()
     
     if 'commentid' in request.GET:
         # redirect to comment URL, this trick is required because some 
@@ -605,30 +622,28 @@ def add_article_comment_ajax(request):
     if len(re.findall('https?://', body, flags=re.IGNORECASE)) <= 1:
         comment.is_approved = True
 
-    article.comments_total += 1
-    if comment.is_approved:
-        article.comments_approved += 1
 
     # record commenter ip address
     comment.ip_address = request.environ.get('REMOTE_ADDR', 'unknown')
     comment.xff_ip_address = request.environ.get('X_FORWARDED_FOR', None)
 
     dbsession.add(comment)
+    _update_comments_counters(dbsession, article)
     dbsession.flush()
     dbsession.expunge(comment) # remove object from the session, object state is preserved
     dbsession.expunge(article)
     transaction.commit()
     
     # comment added, now send notifications
-    cylce_limit = 100
+    loop_limit = 100
     comment = dbsession.query(Comment).get(comment.id)
     parent = comment.parent
     admin_email = get_config('admin_notifications_email')
     vf_q = dbsession.query(VerifiedEmail)
     notifications_emails = list()
     
-    while parent is not None and cylce_limit > 0:
-        cylce_limit -= 1
+    while parent is not None and loop_limit > 0:
+        loop_limit -= 1
         c = parent
         parent = c.parent
         # walk up the tree
@@ -649,7 +664,7 @@ def add_article_comment_ajax(request):
         if email in notifications_emails:
             continue
         
-        vf = vf_q.get(email)
+        vf = vf_q.filter(VerifiedEmail.email==email).first()
         if vf is not None and vf.is_verified:
             # send notification to "email"
             ns.append(notifications.gen_comment_response_notification(request, article, comment, c, email))
@@ -691,11 +706,9 @@ def approve_article_comment_ajax(request):
     if article is None:
         return HTTPNotFound()
 
-    orig = comment.is_approved
     comment.is_approved = True
 
-    if not orig:
-        article.comments_approved += 1
+    _update_comments_counters(dbsession, article)
 
     transaction.commit()
     data = dict()
@@ -708,9 +721,13 @@ def delete_article_comment_ajax(request):
     comment = dbsession.query(Comment).get(comment_id)
     if comment is None:
         return HTTPNotFound()
+
     
     transaction.begin()
     dbsession.delete(comment)
+    article = dbsession.query(Article).get(comment.article_id)
+    _update_comments_counters(dbsession, article)
+
     transaction.commit()
     
     data = dict()
@@ -726,7 +743,7 @@ def edit_fetch_comment_ajax(request):
     
     comment = dbsession.query(Comment).get(comment_id)
     
-    attrs = ('display_name', 'email', 'website', 'body', 'ip_address', 'xff_ip_address')
+    attrs = ('display_name', 'email', 'website', 'body', 'ip_address', 'xff_ip_address', 'is_subscribed')
     data = dict()
     for a in attrs:
         data[a] = getattr(comment, a)
@@ -758,6 +775,7 @@ def edit_article_comment_ajax(request):
         setattr(comment, v, value)
 
     comment.set_body(request.POST['body'])
+    comment.is_subscribed = 'is_subscribed' in request.POST
 
     comment.published = h.str_to_timestamp(request.POST['date'])
     dbsession.flush()
